<?php
/**
 * $Horde: ansel/ecard.php,v 1.10 2005/01/03 14:34:47 jan Exp $
 *
 * Copyright 2004-2005 Michael Slusarz <slusarz@mail.curecanti.org>
 *
 * See the enclosed file COPYING for license information (GPL).  If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 */

@define('ANSEL_BASE', dirname(__FILE__));
require_once ANSEL_BASE . '/lib/base.php';
require_once 'Horde/Form.php';
require_once 'Horde/Form/Renderer.php';
require_once 'Horde/Variables.php';

class EcardForm extends Horde_Form {

    var $_useFormToken = false;

    function EcardForm(&$vars, $title)
    {
        parent::Horde_Form($vars, $title);

        $this->setButtons(_("Send"));
        $this->addHidden('', 'actionID', 'text', false);
        $this->addHidden('', 'gallery', 'text', false);
        $this->addHidden('', 'image', 'text', false);

        $user = Auth::getAuth();
        if (empty($user)) {
            $this->addVariable(_("Use the following return address:"), 'ecard_retaddr', 'text', true);
        } else {
            require_once 'Horde/Identity.php';
            $identity = &Identity::singleton();
            $from_addr = $identity->getDefaultFromAddress();
            $vars->set('ecard_retaddr', $from_addr);
            $this->addHidden('', 'ecard_retaddr', 'text', true);
        }

        $this->addVariable(_("Send ecard to the following address:"), 'ecard_addr', 'text', true);
        $this->addVariable(_("Comments:"), 'ecard_comments', 'longtext', false, false, null, array('15', '60'));
    }

}

$actionID = Util::getFormData('actionID');
$gallery_id = Util::getFormData('gallery');
$image_id = Util::getFormData('image');

/* Close window if ecard sending is disabled or if no action has been
 * requested . */
if (empty($conf['ecard']['enable']) || is_null($actionID)) {
    Util::closeWindowJS();
    exit;
}

/* Close window if no gallery is returned. */
$gallery = &$ansel_shares->getGallery($gallery_id);
if (is_a($gallery, 'PEAR_Error')) {
    Util::closeWindowJS();
    exit;
}

/* Run through the action handlers. */
switch ($actionID) {
case 'send':
    /* Check for required elements. */
    $from = Util::getFormData('ecard_retaddr');
    if (empty($from)) {
        $notification->push(_("You must enter your e-mail address."), 'horde.error');
        break;
    }
    $to = Util::getFormData('ecard_addr');
    if (empty($to)) {
        $notification->push(_("You must enter an e-mail address to send the message to."), 'horde.error');
        break;
    }

    require_once 'Horde/MIME/Headers.php';
    require_once 'Horde/MIME/Message.php';

    /* Set up the mail headers and read the log file. */
    $msg_headers = &new MIME_Headers();
    $msg_headers->addReceivedHeader();
    $msg_headers->addMessageIdHeader();
    $msg_headers->addAgentHeader();
    $msg_headers->addHeader('Date', date('r'));
    $msg_headers->addHeader('From', $from);
    $msg_headers->addHeader('To', $to);
    $msg_headers->addHeader('Subject', _("Ecard - ") . Util::getFormData('image_desc'));

    $image = &$ansel_shares->getImage($image_id);

    $alt = &new MIME_Message();
    $alt->setType('multipart/alternative');

    /* Create the text/plain part that explains to the user that they need
     * to be able to view the Ecard inline. */
    $textpart = &new MIME_Part('text/plain');
    $textpart->setCharset(NLS::getCharset());
    $textpart->setContents(String::wrap(_("You have been sent an Ecard. To view the Ecard, you must be able to view text/html messages in your mail reader. If you are viewing this message, then most likely your mail reader does not support viewing text/html messages.")));
    $alt->addPart($textpart);
    
    /* Create the multipart/related part. */
    $related = &new MIME_Part('multipart/related');

    /* Create the multipart/related part. */
    $htmlpart = &new MIME_Part('text/html');
    $htmlpart->setCharset(NLS::getCharset());
    $imgpart = &new MIME_Part($image->get('type'));
    $imgpart->setContents($image->raw());
    $img_tag = '<img src="cid:' . $imgpart->setContentID() . '" /><p />';
    $comments = $htmlpart->replaceEOL(Util::getFormData('ecard_comments'));
    if (!Util::getFormData('rtemode')) {
        $comments = '<pre>' . htmlspecialchars($comments) . '</pre>';
    }
    $htmlpart->setContents(String::wrap('<html>' . $img_tag . $comments . '</html>'));
    $related->setContentTypeParameter('start', $htmlpart->setContentID());
    $related->addPart($htmlpart);
    $related->addPart($imgpart);

    $alt->addPart($related);

    $msg_headers->addMIMEHeaders($alt);

    $result = $alt->send(Util::getFormData('ecard_addr'), $msg_headers);
    if (is_a($result, 'PEAR_Error')) {
        $actionID = 'query';
        $notification->push(sprintf(_("There was an error sending your message: %s"), $result->getMessage()), 'horde.error');

    } else {
        Util::closeWindowJS();
        exit;
    }
}

$image = &$ansel_shares->getImage($image_id);
if ($image) {
    $filename = $image->get('filename');
    $title = sprintf(_("Send Ecard :: %s"), $filename);

    /* Set up the form object. */
    $vars = &Variables::getDefaultVariables();
    $vars->set('actionID', 'send');
    $form = &Horde_Form::singleton('EcardForm', $vars, $title);
    $renderer = &new Horde_Form_Renderer();

    $gallery = &$ansel_shares->getGallery($image->get('gallery'));

    /* Set up the gallery attributes. */
    $vars->set('image_default', $image->getId() == $gallery->get('default'));
    $img_descrip = $image->get('description');
    if (empty($img_descrip)) {
        $img_descrip = $filename;
    }
    $vars->set('image_desc', $filename);
    $form->addHidden('', 'image_desc', 'text', false);
}

if ($browser->hasFeature('rte')) {
    require_once 'Horde/Editor.php';
    $editor = &Horde_Editor::singleton('htmlarea', array('id' => 'ecard_comments'));
    $vars->set('rtemode', 1);
    $form->addHidden('', 'rtemode', 'text', false);
}

require ANSEL_TEMPLATES . '/common-header.inc';
$notification->notify(array('listeners' => 'status')); 
$form->renderActive($renderer, $vars, 'ecard.php', 'post', 'multipart/form-data');
require $registry->get('templates', 'horde') . '/common-footer.inc';
