<?php
/**
 * $Horde: ansel/image.php,v 1.92 2005/01/03 14:34:47 jan Exp $
 *
 * Copyright 2003-2005 Chuck Hagenbuch <chuck@horde.org>
 *
 * See the enclosed file COPYING for license information (GPL).  If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 */

@define('ANSEL_BASE', dirname(__FILE__));
require_once ANSEL_BASE . '/lib/base.php';
require_once 'Horde/Form.php';
require_once 'Horde/Form/Renderer.php';
require_once 'Horde/Text.php';
require_once 'Horde/Text/Filter.php';
require_once 'Horde/Variables.php';

class ImageForm extends Horde_Form {

    var $_useFormToken = false;

    function ImageForm(&$vars, $title)
    {
        global $gallery;

        parent::Horde_Form($vars, $title);

        $this->setButtons(_("Save"));
        $this->addHidden('', 'actionID', 'text', false);
        $this->addHidden('', 'gallery', 'text', false);
        $this->addHidden('', 'image', 'text', false);
        $this->addHidden('', 'page', 'text', false);
        $this->addHidden('', 'method', 'text', false);

        $this->addVariable(_("Make this the default image for this gallery?"), 'image_default', 'boolean', false);
        $this->addVariable(_("Caption"), 'image_desc', 'longtext', false, false, null, array('4', '40'));
    }

}

class UploadForm extends Horde_Form {

    var $_useFormToken = false;

    function UploadForm(&$vars, $title)
    {
        global $gallery, $conf;

        parent::Horde_Form($vars, $title);

        $this->setButtons(_("Save"));
        $this->addHidden('', 'actionID', 'text', false);
        $this->addHidden('', 'gallery', 'text', false);
        $this->addHidden('', 'page', 'text', false);
        $this->addHidden('', 'method', 'text', false);

        $this->setSection('single_file', _("Single Image"));
        $this->addHidden('', 'image0', 'text', false);
        if (!strlen($vars->get('image0'))) {
            $upload = &$this->addVariable(_("File to upload"), 'file0', 'image', false);
            $upload->setHelp('upload');
        }
        $this->addVariable(_("Make this the default image for this gallery?"), 'image0_default', 'boolean', false);
        $this->addVariable(_("Caption"), 'image0_desc', 'longtext', false, false, null, array('4', '40'));

        $this->setSection('multi_file', _("Multiple Images"));
        for ($i = 1; $i <= $conf['image']['num_uploads']; $i++) { // start at $i = 1 because 0 is used above.
            $this->addHidden('', 'image' . $i, 'text', false);
            if (!strlen($vars->get('image' . $i))) {
                $upload = &$this->addVariable(sprintf(_("File %s"), $i), 'file' . $i, 'image', false, false, null, array(false));
                $upload->setHelp('upload');
            }
        }

        $this->setSection('zip_file', _("Zip File Upload"));
        $this->addHidden('', 'image' . ($conf['image']['num_uploads'] + 1), 'text', false);
        if (!strlen($vars->get('zip'))) {
            $upload = &$this->addVariable(_("File to upload"), 'file' . ($conf['image']['num_uploads'] + 1), 'file', false);
            $upload->setHelp('upload');
        }
    }

}

class WatermarkForm extends Horde_Form {

    var $_useFormToken = false;

    function WatermarkForm(&$vars, $title)
    {
        global $gallery, $prefs;

        parent::Horde_Form($vars, $title);

        $this->setButtons(_("Save"));
        $this->addHidden('', 'actionID', 'text', false);
        $this->addHidden('', 'gallery', 'text', false);
        $this->addHidden('', 'image', 'text', false);
        $this->addHidden('', 'page', 'text', false);

        $w = &$this->addVariable(_("Custom Watermark"), 'watermark', 'text', false, false, null);
        $w->setDefault($prefs->getValue('watermark'));

        $fonts = array('tiny' => _("Tiny"), 'small' => _("Small"), 'medium' => _("Medium"), 'large' => _("Large"), 'giant' => _("Giant"));
        $f = &$this->addVariable(_("Watermark Font"), 'font', 'enum', false, false, null, array($fonts));
        $f->setDefault('small');

        $ha = array('left' => _('Left'), 'center' => _("Center"), 'right' => _("Right"));
        $wha = &$this->addVariable(_("Horizontal Alignment"), 'whalign', 'enum', false, false, null, array($ha));
        $wha->setDefault('right');

        $va = array('top' => _('Top'), 'middle' => _("Middle"), 'bottom' => _("Bottom"));
        $wva = &$this->addVariable(_("Vertical Alignment"), 'wvalign', 'enum', false, false, null, array($va));
        $wva->setDefault('bottom');
    }

}

$actionID = Util::getFormData('actionID');
$gallery_id = Util::getFormData('gallery');
$image_id = Util::getFormData('image');
$page = Util::getFormData('page', 0);
$watermark_font = Util::getFormData('font');
$watermark_halign = Util::getFormData('whalign');
$watermark_valign = Util::getFormData('wvalign');

$watermark = Util::getFormData('watermark', $prefs->getValue('watermark'));
if ($watermark) {
    require_once 'Horde/Identity.php';
    $identity = &Identity::singleton();
    $name = $identity->getValue('fullname');
    if (empty($name)) {
        $name = Auth::getAuth();
    }

    // Set up array of possible substitutions.
    $watermark_array = array('%N' => $name,            // User's fullname.
                             '%L' => Auth::getAuth()); // User login.
    $watermark = str_replace(array_keys($watermark_array), array_values($watermark_array), $watermark);
    $watermark = strftime($watermark);
}

/* Redirect to the image list if no action has been requested. */
if (is_null($actionID)) {
    header('Location: ' . Horde::applicationUrl('list.php', true));
    exit;
}

$gallery = &$ansel_shares->getGallery($gallery_id);
if (is_a($gallery, 'PEAR_Error')) {
    $notification->push(_("Not found or error accessing gallery."), 'horde.error');
    header('Location: ' . Horde::applicationUrl('list.php', true));
    exit;
}

/* Run through the action handlers. */
switch ($actionID) {
case 'add':
    $title = _("Add Image");

    /* Set up the form object. */
    $vars = &Variables::getDefaultVariables();
    $vars->set('actionID', 'save');
    $vars->set('method', 'add');
    $form = &Horde_Form::singleton('UploadForm', $vars, _("Upload new images"));
    $renderer = &new Horde_Form_Renderer();
    break;

case 'modify':
    $image = &$ansel_shares->getImage($image_id);
    if ($image) {
        $title = sprintf(_("Edit properties :: %s"), $image->get('filename'));

        /* Set up the form object. */
        $vars = &Variables::getDefaultVariables();
        $vars->set('actionID', 'saveclose');
        $vars->set('method', 'modify');
        $form = &Horde_Form::singleton('ImageForm', $vars, $title);
        $renderer = &new Horde_Form_Renderer();

        $gallery = &$ansel_shares->getGallery($image->get('gallery'));

        /* Set up the gallery attributes. */
        $vars->set('image_default', $image->getId() == $gallery->get('default'));
        $vars->set('image_desc', $image->get('description'));
    }
    require ANSEL_TEMPLATES . '/common-header.inc';
    $form->renderActive($renderer, $vars, 'image.php', 'post', 'multipart/form-data');
    require $registry->get('templates', 'horde') . '/common-footer.inc';
    exit;

case 'saveclose':
case 'save':
    $title = _("Save Image");

    if (!$gallery->hasPermission(Auth::getAuth(), PERMS_EDIT)) {
        $notification->push(sprintf(_("Access denied saving image to %s."), $gallery->get('name')), 'horde.error');
        $imageurl = Util::addParameter('view.php', array('gallery' => $gallery_id, 'actionID' => 'view', 'page' => $page));
        header('Location: ' . Horde::applicationUrl($imageurl, true));
        exit;
    }

    /* Validate the form object. */
    $vars = &Variables::getDefaultVariables();
    $vars->set('actionID', 'save');
    $method = Util::getFormData('method');

    $renderer = &new Horde_Form_Renderer();

    if ($method == 'modify') {
        $form = &Horde_Form::singleton('ImageForm', $vars, _("Edit an image"));
        /* Update existing image. */
        if ($valid = $form->validate($vars)) {
            $image_id = Util::getFormData('image');
            $image = &$ansel_shares->getImage($image_id);
            $image->set('description', $vars->get('image_desc'), true);

            if ($vars->get('image_default')) {
                $gallery->set('default', $image_id, true);
                $gallery->set('default-type', 'manual', true);
            }
        }
    } else {
        $form = &Horde_Form::singleton('UploadForm', $vars, _("Upload new images"));

        if ($valid = $form->validate($vars)) {
            $form->getInfo($vars, $info);
            for ($i = 0; $i <= $conf['image']['num_uploads'] + 1; $i++) {
                if (!empty($info['file' . $i]['file'])) {
                    /* Save new image. */
                    if (!is_a(Browser::wasFileUploaded('file' . $i), 'PEAR_Error') &&
                        filesize($info['file' . $i]['file'])) {

                        /* Read in the uploaded data. */
                        $fp = fopen($info['file' . $i]['file'], 'r');
                        $data = fread($fp, filesize($info['file' . $i]['file']));
                        fclose($fp);

                        /* Check for a compressed file. */
                        require_once 'Horde/MIME/Magic.php';
                        if (in_array($info['file' . $i]['type'],
                                     array('x-extension/zip',
                                           'application/x-compressed',
                                           'application/x-zip-compressed',
                                           'application/zip')) ||
                            MIME_Magic::filenameToMIME($info['file' . $i]['name']) == 'application/zip') {

                            /* Get the list of files in the zipfile. */
                            require_once 'Horde/Compress.php';
                            $zip = &Horde_Compress::singleton('zip');
                            $files = $zip->decompress($data, array('action' => HORDE_COMPRESS_ZIP_LIST));

                            if (is_a($files, 'PEAR_Error')) {
                                $notification->push(sprintf(_("There was an error processing the uploaded archive: %s"), $files->getMessage()), 'horde.error');
                            } else {
                                $success = 0;
                                foreach ($files as $key => $zinfo) {
                                    $zdata = $zip->decompress($data, array('action' => HORDE_COMPRESS_ZIP_DATA,
                                                                           'info' => $files,
                                                                           'key' => $key));
                                    if (is_a($zdata, 'PEAR_Error')) {
                                        $notification->push(sprintf(_("There was an error processing the uploaded archive: %s"), $zdata->getMessage()), 'horde.error');
                                        break;
                                    }

                                    /* If we successfully got data,
                                     * try adding the image to the
                                     * gallery. */
                                    $image = array('filename' => $zinfo['name'],
                                                   'description' => '',
                                                   'data' => $zdata,
                                                   );
                                    $image_id = $gallery->addImage($image);
                                    if (!is_a($image_id, 'PEAR_Error')) {
                                        $success++;
                                    } else {
                                        $notification->push(sprintf(_("There was a problem saving the image: %s"), $image_id), 'horde.error');
                                    }
                                }

                                /* Tell the user how well we did. */
                                if ($success) {
                                    if ($success == 1) {
                                        $notification->push(_("1 image was successfully extracted and uploaded."), 'horde.success');
                                    } else {
                                        $notification->push(sprintf(_("%d images were successfully extracted and uploaded."), $success), 'horde.success');
                                    }
                                }
                            }
                        } else {
                            /* Try and make sure the image is in a
                             * recognizeable format. */
                            if (getimagesize($info['file' . $i]['file']) === false) {
                                $notification->push(_("The file you uploaded does not appear to be a valid image."), 'horde.error');
                            } else {
                                $image = array('filename' => $info['file' . $i]['name'],
                                               'description' => $vars->get('image' . $i . '_desc'),
                                               'data' => $data,
                                               'type' => $info['file' . $i]['type'],
                                               );

                                $image_id = $gallery->addImage($image);
                                if (!is_a($image_id, 'PEAR_Error')) {
                                    $notification->push(_("The image was saved."), 'horde.success');
                                    if ($vars->get('image_default')) {
                                        $gallery->set('default', $image_id, true);
                                        $gallery->set('default-type', 'manual', true);
                                    }
                                } else {
                                    $notification->push(sprintf(_("There was a problem saving the image: %s"), $image_id->getMessage()), 'horde.error');
                                    $valid = false;
                                }
                            }
                        }
                    } else {
                        if (!empty($info['file' . $i]['error'])) {
                            $notification->push(sprintf(_("There was a problem uploading the image: %s"), $info['file' . $i]['error']), 'horde.error');
                        } elseif (!filesize($info['file' . $i]['file'])) {
                            $notification->push(_("The uploaded file appears to be empty. It may not exist on your computer."), 'horde.error');
                        }
                        $valid = false;
                    }
                }
            }
        }
    }

    if ($valid) {
        if ($actionID == 'save') {
            /* Return to the gallery view. */
            $imageurl = Util::addParameter('view.php', array('gallery' => $gallery_id, 'image' => $image_id, 'actionID' => 'view', 'page' => $page));
            header('Location: ' . Horde::applicationUrl($imageurl, true));
        } else {
            Util::closeWindowJS(' window.opener.location.href = window.opener.location.href;');
        }
        exit;
    }
    break;

case 'exif':
    $image = &$ansel_shares->getImage($image_id);
    $title = sprintf(_("Attributes for %s :: %s"), $gallery->get('name'), $image->get('filename'));
    require_once ANSEL_BASE . '/lib/Exif.php';
    require ANSEL_TEMPLATES . '/common-header.inc';
    require ANSEL_TEMPLATES . '/image/exif.inc';
    require $registry->get('templates', 'horde') . '/common-footer.inc';
    exit;

case 'editimage':
    if (!$gallery->hasPermission(Auth::getAuth(), PERMS_EDIT)) {
        $notification->push(_("You are not allowed to edit this image."), 'horde.error');

        /* Return to the image view. */
        $imageurl = Util::addParameter('view.php', array('gallery' => $gallery_id, 'image' => $image_id, 'actionID' => 'view', 'page' => $page));
        header('Location: ' . Horde::applicationUrl($imageurl, true));
        exit;
    }

    /* Retrieve image details. */
    $image = &$ansel_shares->getImage($image_id);
    $title = sprintf(_("Edit %s :: %s"), $gallery->get('name'), $image->get('filename'));

    require ANSEL_TEMPLATES . '/common-header.inc';
    require ANSEL_TEMPLATES . '/menu.inc';
    require ANSEL_TEMPLATES . '/image/edit_image.inc';
    require $registry->get('templates', 'horde') . '/common-footer.inc';
    exit;

case 'rotate90':
case 'rotate180':
case 'rotate270':
case 'flip':
case 'mirror':
case 'grayscale':
case 'watermark':
    if (!$gallery->hasPermission(Auth::getAuth(), PERMS_EDIT)) {
        $notification->push(sprintf(_("Access denied saving image to %s."), $gallery->get('name')), 'horde.error');
    } else {
        $image = &$ansel_shares->getImage($image_id);

        switch ($actionID) {
        case 'rotate90':
        case 'rotate180':
        case 'rotate270':
            $angle = intval(substr($actionID, 6));
            $image->rotate($angle);
            break;

        case 'flip':
            $image->flip();
            break;

        case 'mirror':
            $image->mirror();
            break;

        case 'grayscale':
            $image->grayscale();
            break;

        case 'watermark':
            $image->watermark($watermark, $watermark_halign, $watermark_valign, $watermark_font);
            break;
        }

        $image->updateData($image->raw());
    }

    $imageurl = Util::addParameter('image.php', array('gallery' => $gallery_id, 'image' => $image_id, 'actionID' => 'editimage', 'page' => $page));
    header('Location: ' . Horde::applicationUrl($imageurl, true));
    exit;

case 'setwatermark':
    $title = _("Watermark");
    $image = &$ansel_shares->getImage($image_id);
    if ($image) {
        /* Set up the form object. */
        $vars = &Variables::getDefaultVariables();
        $vars->set('actionID', 'previewcustomwatermark');
        $form = &Horde_Form::singleton('WatermarkForm', $vars, _("Watermark"));
        $renderer = &new Horde_Form_Renderer();

        $gallery = &$ansel_shares->getGallery($image->get('gallery'));
    }
    require ANSEL_TEMPLATES . '/common-header.inc';
    $form->renderActive($renderer, $vars, 'image.php', 'post');
    require $registry->get('templates', 'horde') . '/common-footer.inc';
    exit;

case 'previewcustomwatermark':
    $imageurl = Util::addParameter('image.php', array('gallery' => $gallery_id, 'image' => $image_id, 'page' => $page, 'watermark' => $watermark, 'font' => $watermark_font, 'whalign' => $watermark_halign, 'wvalign' => $watermark_valign, 'actionID' => 'previewwatermark'));
    $url = Horde::applicationUrl($imageurl);
    $url = str_replace('&amp;', '&', $url);
    Util::closeWindowJS('window.opener.location.href = "' . $url . '";');
    exit;

case 'previewgrayscale':
case 'previewwatermark':
case 'previewflip':
case 'previewmirror':
case 'previewrotate90':
case 'previewrotate180':
case 'previewrotate270':
    $title = _("Edit Image");
    $action = substr($actionID, 7);

    /* If the requested gallery doesn't exist, display an error
     * message. */
    if (is_a($gallery, 'PEAR_Error')) {
        $notification->push(_("Gallery not found."), 'horde.error');
        header('Location: ' . Horde::applicationUrl('list.php', true));
        exit;
    } else {
        /* Retrieve image details. */
        $image = &$ansel_shares->getImage($image_id);
        $title = sprintf(_("Preview changes for %s :: %s"), $gallery->get('name'), $image->get('filename'));

        require ANSEL_TEMPLATES . '/common-header.inc';
        require ANSEL_TEMPLATES . '/menu.inc';
        require ANSEL_TEMPLATES . '/image/preview_image.inc';
        require $registry->get('templates', 'horde') . '/common-footer.inc';
        exit;
    }
    break;

case 'imagerotate90':
case 'imagerotate180':
case 'imagerotate270':
    $angle = intval(substr($actionID, 11));
    $image = &$ansel_shares->getImage($image_id);
    $image->rotate($angle);
    $image->display();
    exit;

case 'imageflip':
    $image = &$ansel_shares->getImage($image_id);
    $image->flip();
    $image->display();
    exit;

case 'imagemirror':
    $image = &$ansel_shares->getImage($image_id);
    $image->mirror();
    $image->display();
    exit;

case 'imagegrayscale':
    $image = &$ansel_shares->getImage($image_id);
    $image->grayscale();
    $image->display();
    exit;

case 'imagewatermark':
    $image = &$ansel_shares->getImage($image_id);
    $image->watermark($watermark, $watermark_halign, $watermark_valign, $watermark_font);
    $image->display();
    exit;

case 'imagefull':
case 'imagescreen':
case 'imagethumb':
case 'imagemini':
    $view = substr($actionID, 5);
    $image = &$ansel_shares->getImage($image_id);
    $image->display($view);
    exit;

case 'downloadfull':
case 'downloadscreen':
case 'downloadthumb':
    $image = &$ansel_shares->getImage($image_id);
    $view = substr($actionID, 8);

    /* Send downloading headers. */
    $image->downloadHeaders($view);

    /* Send the image data. */
    $image->display($view);
    exit;

case 'delete':
    if (!is_array($image_id)) {
        $image_id = array($image_id => true);
    }

    /* Delete the images if we're provided with a valid image ID. */
    if (count($image_id)) {
        foreach($image_id as $image => $delete) {
            if ($delete) {
                if (!$gallery->hasPermission(Auth::getAuth(), PERMS_DELETE)) {
                    $notification->push(sprintf(_("Access denied deleting image %s."), $image), 'horde.error');
                } else {
                    $result = $gallery->removeImage($image, true);
                    if (!is_a($result, 'PEAR_Error')) {
                        $notification->push(_("Deleted the image."), 'horde.success');
                    } else {
                        $notification->push(sprintf(_("There was a problem deleting images: %s"), $result->getMessage()), 'horde.error');
                    }
                }
            }
        }
    }

    /* Return to the image list. */
    $imageurl = Util::addParameter('view.php', array('gallery' => $gallery_id, 'actionID' => 'view', 'page' => $page));
    header('Location: ' . Horde::applicationUrl($imageurl, true));
    exit;

case 'move':
    if (!is_array($image_id)) {
        $image_id = array($image_id => true);
    }

    /* Move the images if we're provided with a valid image ID. */
    if (count($image_id)) {
        $newGallery0 = Util::getFormData('new_gallery0');
        $newGallery1 = Util::getFormData('new_gallery1');
        $newGallery = $newGallery0 == DATATREE_ROOT ? $newGallery1 : $newGallery0;
        if ($newGallery != DATATREE_ROOT) {
            $newGallery = &$ansel_shares->getGallery($newGallery);
            foreach($image_id as $image => $move) {
                if ($move) {
                    if (!$gallery->hasPermission(Auth::getAuth(), PERMS_DELETE)) {
                        $notification->push(sprintf(_("Access denied deleting image %s."), $image), 'horde.error');
                    } else {
                        $imageOb = &$ansel_shares->getImage($image);
                        $result1 = $newGallery->addImage(array('description' => $imageOb->get('desc'),
                                                    'data' => $imageOb->raw(),
                                                    'filename' => $imageOb->get('filename'),
                                                    'type' => $imageOb->get('type')));
                        $result2 = $gallery->removeImage($imageOb, true);
                        if (!is_a($result1, 'PEAR_Error') && !is_a($result2, 'PEAR_Error')) {
                            $notification->push(sprintf(_("Moved %s to %s."), $imageOb->get('filename'), $newGallery->get('name')), 'horde.success');
                        } else if (is_a($result1, 'PEAR_Error')) {
                            $notification->push(sprintf(_("There was a problem moving image: %s"), $result1->getMessage()), 'horde.error');
                        } else {
                            $notification->push(sprintf(_("There was a problem moving image: %s"), $result2->getMessage()), 'horde.error');
                        }
                    }
                }
            }
        }
    }

    /* Return to the image list. */
    $imageurl = Util::addParameter('view.php', array('gallery' => $gallery_id, 'actionID' => 'view', 'page' => $page));
    header('Location: ' . Horde::applicationUrl($imageurl, true));
    exit;

case 'copy':
    if (!is_array($image_id)) {
        $image_id = array($image_id => true);
    }

    /* Copy the images if we're provided with a valid image ID. */
    if (count($image_id)) {
        $newGallery0 = Util::getFormData('new_gallery0');
        $newGallery1 = Util::getFormData('new_gallery1');
        $newGallery = $newGallery0 == DATATREE_ROOT ? $newGallery1 : $newGallery0;
        if ($newGallery != DATATREE_ROOT) {
            $newGallery = &$ansel_shares->getGallery($newGallery);
            foreach ($image_id as $image => $move) {
                if ($move) {
                    $imageOb = &$ansel_shares->getImage($image);
                    $result = $newGallery->addImage(array('description' => $imageOb->get('desc'),
                                                          'data' => $imageOb->raw(),
                                                          'filename' => $imageOb->get('filename'),
                                                          'type' => $imageOb->get('type')));
                    if (!is_a($result, 'PEAR_Error')) {
                        $notification->push(sprintf(_("Copied %s to %s."), $imageOb->get('filename'), $newGallery->get('name')), 'horde.success');
                    } else {
                        $notification->push(sprintf(_("There was a problem copying image: %s"), $result->getMessage()), 'horde.error');
                    }
                }
            }
        }
    }

    /* Return to the image list. */
    $imageurl = Util::addParameter('view.php', array('gallery' => $gallery_id, 'actionID' => 'view', 'page' => $page));
    header('Location: ' . Horde::applicationUrl($imageurl, true));
    exit;

default:
    header('Location: ' . Horde::applicationUrl('list.php', true));
    exit;
}

require ANSEL_TEMPLATES . '/common-header.inc';
require ANSEL_TEMPLATES . '/menu.inc';
$form->renderActive($renderer, $vars, 'image.php', 'post', 'multipart/form-data');
require $registry->get('templates', 'horde') . '/common-footer.inc';
