<?php
/**
 * $Horde: ansel/list.php,v 1.58 2005/01/03 14:34:47 jan Exp $
 *
 * Copyright 2001-2005 Chuck Hagenbuch <chuck@horde.org>
 *
 * See the enclosed file COPYING for license information (GPL).  If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 */

@define('ANSEL_BASE', dirname(__FILE__));
require_once ANSEL_BASE . '/lib/base.php';

/* Sort out the sorting values and gallery filtering. */
if (($sortby = Util::getFormData('sortby')) !== null) {
    $prefs->setValue('sortby', $sortby);
}
if (($sortdir = Util::getFormData('sortdir')) !== null) {
    $prefs->setValue('sortdir', $sortdir);
}

/* If we aren't supplied with a page number, default to page 0. */
$gpage = Util::getFormData('gpage', 0);
$galleries_perpage = $prefs->getValue('galleriesperpage');

/* See if an owner is specified. */
$owner = Util::getFormData('owner');

$num_galleries = Ansel::countGalleries(PERMS_SHOW, $owner, DATATREE_ROOT, false);
if ($num_galleries == 0) {
    $notification->push(_("There are no photo galleries available."), 'horde.message');
    $gallerylist = array();
} else {
    $gallerylist = Ansel::listGalleries(PERMS_SHOW, $owner, DATATREE_ROOT, false, $gpage * $galleries_perpage, $galleries_perpage);
}

require_once 'Horde/UI/Pager.php';
require_once 'Horde/Variables.php';
$vars = &Variables::getDefaultVariables();
$gallery_pager = &new Horde_UI_Pager('gpage', $vars, array('num' => $num_galleries,
                                                           'url' => Util::addParameter('list.php', 'owner', $owner),
                                                           'perpage' => $galleries_perpage));

if ($prefs->getValue('listtype')) {
    Horde::addScriptFile('hideable.js', 'horde', true);
    Horde::addScriptFile('listtiles.js', 'ansel');
}
require ANSEL_TEMPLATES . '/common-header.inc';
require ANSEL_TEMPLATES . '/menu.inc';

if ($num_galleries) {
    foreach ($gallerylist as $groupId => $group) {
        $title = $group['title'];
        $galleries = $group['galleries'];

        switch ($prefs->getValue('listtype')) {
        case 'tile':
            $title .= ' (' . count($group['galleries']) . ')';
            require ANSEL_TEMPLATES . '/list/tile.inc';
            break;

        case 'list':
        default:

            $min = $gpage * $galleries_perpage;
            $max = $min + $galleries_perpage;
            if ($max > $num_galleries) {
                $max = $num_galleries - $min;
            }
            $start = $min + 1;
            $end = min($num_galleries, $min + $galleries_perpage);

            require ANSEL_TEMPLATES . '/list/header.inc';
            $count = 0;
            foreach ($group['galleries'] as $galleryId => $gallery) {
                $viewurl = Horde::applicationUrl(Util::addParameter('view.php', 'gallery', $gallery->getId()));
                $galleryurl = Util::addParameter('gallery.php', 'gallery', $gallery->getId());
                $ownerurl = Horde::applicationUrl(Util::addParameter('list.php', 'owner', $gallery->get('owner')));
                require ANSEL_TEMPLATES . '/list/images.inc';
            }
            while ($count++ % $prefs->getValue('imagesperrow')) {
                require ANSEL_TEMPLATES . '/list/filler.inc';
            }
            require ANSEL_TEMPLATES . '/list/footer.inc';
            break;
        }
    }
    require ANSEL_TEMPLATES . '/list/pager.inc';
}

$title = _("Gallery List");
require $registry->get('templates', 'horde') . '/common-footer.inc';
