<?php
/**
 * $Horde: ansel/reorder.php,v 1.14 2005/01/03 14:34:47 jan Exp $
 *
 * Copyright 2001-2005 Chuck Hagenbuch <chuck@horde.org>
 *
 * See the enclosed file COPYING for license information (GPL).  If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 */

@define('ANSEL_BASE', dirname(__FILE__));
require_once ANSEL_BASE . '/lib/base.php';
require_once 'Horde/Form.php';
require_once 'Horde/Form/Renderer.php';

class ReorderForm extends Horde_Form {

    var $_useFormToken = false;

    function ReorderForm(&$vars, $title)
    {
        global $gallery, $prefs;

        parent::Horde_Form($vars, $title);

        $this->setButtons(_("Save"));
        $this->addHidden('', 'actionID', 'text', false);
        $this->addHidden('', 'gallery', 'text', false);
        $this->addHidden('', 'image', 'text', false);
        $this->addHidden('', 'page', 'text', false);
    }
}


/* If we aren't provided with a gallery, redirect to the gallery
 * list. */
$galleryId = Util::getFormData('gallery');
$actionID = Util::getFormData('actionID');

if (!isset($galleryId)) {
    header('Location: ' . Horde::applicationUrl('list.php', true));
    exit;
}

$gallery = &$ansel_shares->getGallery($galleryId);
if (is_a($gallery, 'PEAR_Error') || !$gallery->hasPermission(Auth::getAuth(), PERMS_READ)) {
    $notification->push(_("You do not have permission to view this gallery."), 'horde.error');
    header('Location: ' . Horde::applicationUrl('list.php', true));
    exit;
}

require_once 'Horde/Variables.php';

$viewurl = Util::addParameter('view.php', 'gallery', $galleryId);
$vars = &Variables::getDefaultVariables();

$title = $gallery->get('name');

/* See if we have any parents to go up to. */
$parent_list = $gallery->getParents();
$parents = array();
foreach($parent_list as $name) {
    $p = $ansel_shares->getGallery($name);
    $parents[] = Horde::link(Horde::applicationUrl(Util::addParameter('view.php', 'gallery', $p->getName())), $p->get('name'), 'smallheader') . $p->get('name') . '</a>';
}

$imagelist = $gallery->listImages();

switch ($actionID) {
case 'setOrder':
    $order = array();
    foreach (array_keys($imagelist) as $id) {
        $gallery->setImageOrder($id, Util::getFormData('o_' . $id));
    }

    $notification->push(_("Gallery re-ordered."), 'horde.success');
    header('Location: ' . Horde::applicationUrl(Util::addParameter('view.php', 'gallery', $galleryId), true));
    exit;
}

require ANSEL_TEMPLATES . '/common-header.inc';
require ANSEL_TEMPLATES . '/menu.inc';

if (count($parents)) {
    $parents = array_reverse($parents);
    require ANSEL_TEMPLATES . '/view/parents.inc';
}

$vars->set('actionID', 'setOrder');
$form = &Horde_Form::singleton('ReorderForm', $vars, sprintf(_("Re-order the images in %s"), $title));

$num = 0;
foreach ($imagelist as $id => $name) {
    $num++;
    $image = &$ansel_shares->getImage($id);
    $preview = '';

    $preview .= '<script type="text/javascript" language="Javascript">';
    $preview .= 'img' . $id . 'status = \'mini\';';
    $preview .= 'function toggle' . $id . '() {';
    $preview .= 'if (img' . $id . 'status == \'mini\') {';
    $preview .= 'document.img' . $id . '.src=\'' . Ansel::getImageUrl($id, 'thumb') . '\';';
    $preview .= 'img' . $id . 'status = \'thumb\';';
    $preview .= '} else {';
    $preview .= 'document.img' . $id . '.src=\'' . Ansel::getImageUrl($id, 'mini') . '\';';
    $preview .= 'img' . $id . 'status = \'mini\';';
    $preview .= '}}';
    $preview .= '</script>';
    $preview .= '<div class="dropShadow" style="float:right">';
    $preview .= Horde::img(Ansel::getImageUrl($id, 'mini'), '', 'style="cursor:pointer" id="img' . $id . '" onclick="toggle' . $id . '();"', '');
    $preview .= '</div>';
    $preview .= '<div>';
    $preview .= $image->get('description');
    $preview .= '</div>';

    $o = &$form->addVariable($preview, 'o_' . $id, 'text', true, false, null, array('[:digit]', '3', '3'));
    $o->setDefault($image->get('order') ? $image->get('order') : $num);
}

$renderer = &new Horde_Form_Renderer();
$renderer->setAttrColumnWidth('75%');
$form->renderActive($renderer, $vars, 'reorder.php', 'post', 'multipart/form-data');

require $registry->get('templates', 'horde') . '/common-footer.inc';
