<?php
/**
 * $Horde: ansel/search.php,v 1.13 2005/01/03 14:34:47 jan Exp $
 *
 * Copyright 2001-2005 Chuck Hagenbuch <chuck@horde.org>
 *
 * See the enclosed file COPYING for license information (GPL).  If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 */

@define('ANSEL_BASE', dirname(__FILE__));
require_once ANSEL_BASE . '/lib/base.php';
require_once ANSEL_BASE . '/lib/Exif.php';
require_once 'Horde/Form.php';
require_once 'Horde/Form/Renderer.php';
require_once 'Horde/Form/Action.php';
require_once 'Horde/Template.php';
require_once 'Horde/Variables.php';

class SearchForm extends Horde_Form {

    var $_useFormToken = false;

    function SearchForm(&$vars, $title)
    {
        parent::Horde_Form($vars, $title);

        $this->setButtons(_("Search"));
        $this->addHidden('', 'actionID', 'text', false);

        // This gets the known Exif fields.
        $fields = Ansel_ImageData::getFields();

        // Now add on any others that we may want.
        $fields += array('description' => array('description' => _("Description"), 'type' => 'text'),
                         'date-uploaded' => array('description' => _("Date Uploaded"), 'type' => 'date'),
                         'filename' => array('description' => _("Filename"), 'type' => 'text'),
                         'gallery' => array('description' => _("Gallery Name"), 'type' => 'text'));

        // reformat the array so that the Form Renderer likes it
        // amd grab only the non-date based fields
        $searchFields = array();
        foreach ($fields as $field => $data) {
            if ($data['type'] != 'date') {
                $searchFields += array($field => $data['description']);
            }
        }

        asort($searchFields);

        $ops = array('=' => _("Is"),
                     '!=' => _("Is Not"),
                     'BEGIN' => _("Begins With"),
                     'END' => _("Ends With"),
                     'LIKE' => _("Contains"),
                     'NOT LIKE' => _("Does Not Contain"));

        // Add the select box to tell us what type of search to
        // perform.
        $this->addVariable(_("Search Type"), 'searchType', 'enum', false, false, null,
                           array(array('TEXT' => _("Text Fields"), 'DATE' => _("Date Fields"))));

        $this->addVariable(null, null, 'spacer', false, false, null);

        // Add the text search form items.
        $this->addVariable(_("Text Field"), 'textField', 'enum', false, false, null, array($searchFields));
        $this->addVariable(_("Text Operator"), 'textOp', 'enum', false, false, null, array($ops));
        $this->addVariable(_("Text Value"), 'textValue', 'text', false, false, null);
        $this->addVariable(null, null, 'spacer', false, false, null);

        // Reformat the array so that the Form Renderer likes it and
        // grab only the date based fields.
        $searchFields = array();
        foreach ($fields as $field => $data) {
            if ($data['type'] == 'date') {
                $searchFields[$field] = $data['description'];
            }
        }

        asort($searchFields);

        $ops = array('BETWEEN' => _("Between"),
                     '>' => _("After"),
                     '<' => _("Before"));

        $this->addVariable(_("Date Field"), 'dateField', 'enum', false, false, null, array($searchFields));
        $this->addVariable(_("Date Operator"), 'dateOp', 'enum', false, false, null, array($ops));
        $this->addVariable(_("Start Date"), 'startDate', 'monthDayYear', false, false, null);
        $this->addVariable(_("End Date"), 'endDate', 'monthDayYear', false, false, null);
    }

}

$actionID = Util::getFormData('actionID');
$searchType = Util::getFormData('searchType');
$vars = &Variables::getDefaultVariables();
$title = _("Search for Images");
require ANSEL_TEMPLATES . '/common-header.inc';
require ANSEL_TEMPLATES . '/menu.inc';

$vars->set('actionID', 'search');
$form = &Horde_Form::singleton('SearchForm', $vars, sprintf(_("Search for Images"), $title));

// This should be automatically included by the setAction() calls
// above, but it's not, so let's include it here manually.
Horde::addScriptFile('form_helpers.js', 'horde', true);

$renderer = &new Horde_Form_Renderer();
$form->renderActive($renderer, $vars, 'search.php', 'post', 'multipart/form-data');

// Now display any results, if a search was performed.
switch ($actionID) {
case 'search':
    switch ($searchType) {
    case 'TEXT':
        // Set up the search criteria to use.
        $attr_name = Util::getFormData('textField');
        $attr_op = Util::getFormData('textOp');
        $attr_value = Util::getFormData('textValue');

        // Escape any SQL wildcards.
        $attr_value = str_replace(array('%', '_'), array('\%', '\_'), $attr_value);

        // For the CONTAINS and DOES NOT CONTAIN cases, we add the SQL
        // regex wildcards to the front and back of the value.
        if ($attr_op == 'LIKE' || $attr_op == 'NOT LIKE') {
            $attr_value = '%' . $attr_value . '%';
        } elseif ($attr_op == 'BEGIN') {
            $attr_op = 'LIKE';
            $attr_value = $attr_value . '%';
        } elseif ($attr_op == 'END') {
            $attr_op = 'LIKE';
            $attr_value = '%' . $attr_value;
        }

        $criteria['AND'] = array(array('field' => 'name',
                                       'op' => '=',
                                       'test' => $attr_name),
                                 array('field' => 'value',
                                       'op' => $attr_op,
                                       'test' => $attr_value));

        $imagelist = $ansel_shares->searchAllImages($criteria);

        // Now display the results.
        Horde::addScriptFile('popup.js', 'horde', true);

        $i = 0;
        $images = array();
        foreach ($imagelist as $id => $name) {
            $image = &$ansel_shares->getImage($id);
            $galleryName = $image->get('gallery');
            $gallery = $ansel_shares->getGallery($galleryName);
            // Check that the user has permission to view this image
            // before displaying the link.
            if (is_a($gallery, 'PEAR_Error') || !$gallery->hasPermission(Auth::getAuth(), PERMS_READ)) {
                continue;
            }

            $img = Util::addParameter('image.php', array('gallery' => $image->get('gallery'),
                                                         'image' => $image->getId(),
                                                         'actionID' => 'imagethumb'));
            $images[] = array('i' => ($i++ %2),
                              'image' => Horde::link('', _("Thumbnail"), '', '', 'popup(\'' . Horde::applicationUrl($img) . '\',' . ($conf['thumbnail']['width'] + 10) . ', ' . ($conf['thumbnail']['height'] + 10) . '); return false;') . $image->get('filename') . '</a>',
                              'gallery' => Horde::link(Horde::applicationUrl(Util::addParameter('view.php', array('gallery' => $galleryName, 'page' => '1')))) . $gallery->get('name') . '</a>',
                              'description' => $image->get('description') ? htmlspecialchars($image->get('description')) : '&nbsp;');
        }

        $template = &new Horde_Template();
        $template->set('images', $images);
        $template->set('heading', count($images) == 1 ? _("Matched 1 image") : sprintf(_("Matched %s images"), count($images)));
        echo $template->fetch(ANSEL_TEMPLATES . '/search/search.html');
        break;

    case 'DATE':
        $attr_name = Util::getFormData('dateField');
        $attr_op = Util::getFormData('dateOp');
        $attr_startDate = Util::getFormData('startDate');
        $attr_endDate = Util::getFormData('endDate');

        // Convert the date form fields to a timestamp.
        $startDate = strtotime(sprintf("%s/%s/%s 00:00:00", $attr_startDate['month'], $attr_startDate['day'], $attr_startDate['year']));
        $endDate = strtotime(sprintf("%s/%s/%s 00:00:00", $attr_endDate['month'], $attr_endDate['day'], $attr_endDate['year']));

        // Set up the proper search criteria.
        $criteria = array();
        if ($attr_op == "BETWEEN") {
            $criteria['AND'] = array(array('AND' => array(array('field' => 'name',
                                                                'op' => '=',
                                                                'test' => $attr_name),
                                                          array('field' => 'value',
                                                                'op' => '>',
                                                                'test' => $startDate))),
                                     array('AND' => array(array('field' => 'name',
                                                                'op' => '=',
                                                                'test' => $attr_name),
                                                          array('field' => 'value',
                                                                'op' => '<',
                                                                'test' => $endDate))));
        } else {
            $criteria['AND'] = array(array('field' => 'name',
                                           'op' => '=',
                                           'test' => $attr_name),
                                     array('field' => 'value',
                                           'op' => $attr_op,
                                           'test' => $startDate));
        }

        // Actually do the search.
        $imagelist = $ansel_shares->searchAllImages($criteria);

        // Now display the results.
        Horde::addScriptFile('popup.js', 'horde', true);

        $i = 0;
        $images = array();
        foreach ($imagelist as $id => $name) {
            $image = &$ansel_shares->getImage($id);
            $galleryName = $image->get('gallery');
            $gallery = $ansel_shares->getGallery($galleryName);
            // Check that the user has permission to view this image
            // before displaying the link.
            if (is_a($gallery, 'PEAR_Error') || !$gallery->hasPermission(Auth::getAuth(), PERMS_READ)) {
                continue;
            }

            $img = Util::addParameter('image.php', array('gallery' => $image->get('gallery'),
                                                         'image' => $image->getId(),
                                                         'actionID' => 'imagethumb'));
            $images[] = array('i' => ($i++ %2),
                              'image' => Horde::link('', _("Thumbnail"), '', '', 'popup(\'' . Horde::applicationUrl($img) . '\',' . ($conf['thumbnail']['width'] + 10) . ', ' . ($conf['thumbnail']['height'] + 10) . '); return false;') . $image->get('filename') . '</a>',
                              'gallery' => Horde::link(Horde::applicationUrl(Util::addParameter('view.php', array('gallery' => $galleryName, 'page' => '1')))) . $gallery->get('name') . '</a>',
                              'description' => $image->get('description') ? htmlspecialchars($image->get('description')) : '&nbsp;');
        }

        $template = &new Horde_Template();
        $template->set('images', $images);
        $template->set('heading', count($images) == 1 ? _("Matched 1 image") : sprintf(_("Matched %s images"), count($images)));
        echo $template->fetch(ANSEL_TEMPLATES . '/search/search.html');
        break;
    }
}

require $registry->get('templates', 'horde') . '/common-footer.inc';
